import logging
import sys
import os
class Logger(object):

    def __init__(self,logname,stream_level="info",file_level="debug"):
        self.logname = logname
        self.logger = logging.getLogger(logname)
        self.logger.setLevel(level=logging.DEBUG)
        self.add_stream_hander(stream_level)
        self.add_file_hander(file_level)
        self.add_method()

    def get_level(self,level):
        if level == "info":
            return logging.INFO
        elif level == "debug":
            return logging.DEBUG
        elif level == "warning":
            return logging.WARNING

    def add_method(self):
        self.info = self.logger.info
        self.debug = self.logger.debug
        self.warning = self.logger.warning
        self.error = self.logger.error

    def add_stream_hander(self,level):
        level = self.get_level(level)
        stream_handler = logging.StreamHandler(sys.stdout)
        stream_handler.setLevel(level=level)
        self.logger.addHandler(stream_handler)

    def add_file_hander(self,level):
        if not os.path.exists('log'):
            os.mkdir('log')
        level = self.get_level(level)
        file_handler = logging.FileHandler('log/{}.log'.format(self.logname))
        file_handler.setLevel(level=level)
        formatter = logging.Formatter('[%(asctime)s %(levelname)s] %(message)s')
        file_handler.setFormatter(formatter)
        self.logger.addHandler(file_handler)

if __name__ == '__main__':
    logger  = Logger("gui")
    logger.info("hello")
    logger.warning("hello")


from selenium.webdriver.support.select import Select
from selenium.webdriver.common.keys import Keys
from .common import timethis

class WebElement(object):
    """
    基于selenium的页面方法功能，添加或修改了被选中的页面元素的方法
    """
    def __init__(self,selenium_elem,selenium_driver):
        """

        """
        self.driver = selenium_driver
        self.elem = selenium_elem

    @timethis
    def select(self,text=None,index=None,value=None):
        if text:
            print('SELECT OPTION BY TEXT : {}'.format(text),end = ' ')
            Select(self.elem).select_by_visible_text(text)

        elif index:
            print('SELECT OPTION BY INDEX : {}'.format(index),end = ' ')
            Select(self.elem).select_by_visible_text(index)

        elif value:
            print('SELECT OPTION BY VALUE : {}'.format(value),end = ' ')
            Select(self.elem).select_by_visible_text(value)

    @timethis
    def send_keys(self,text,isclear=True):
        '''
        向文本框发送文本
        ARGS:
            * ISCLEAR [bool] 发送文本前，是否清空输入框
        '''
        print("SEND KEYS : {}".format(text),end = ' ')
        self.elem.click()
        if isclear:
            self.driver.execute_script("arguments[0].value = '';", self.elem)
        self.elem.send_keys(text)

    @timethis
    def check(self,ischeck=True):
        '''
        勾选/反选 raido/checkbox 元素
        ARGS:
            * ISCHECK [bool] 勾选/反选被选中的元素
                * True  勾选被选中的元素 [default]
                * False 反选被选中的元素
        '''

        if ischeck:
            print("CHECK ELEM",end = ' ')
            if not self.elem.is_selected():
                self.elem.click()
        else:
            print("UNCHECK ELEM",end = ' ')
            if self.elem.is_selected():
                self.elem.click()

    @timethis
    def click(self):
        """
        点击一个元素
        """
        print('CLICK ELEM',end = ' ')
        for i in range(30):
            try:
                self.elem.click()
                return True
            except Exception as e:
                self.save_screenshot('ERROR.png')
            time.sleep(1)
        raise Exception("CAN'T CLICK")

    def scroll(self,method):
        """
        ARGS:
            *  METHOD
                * pageup 向上一页滚动
                * pagedown 向下一页滚动
        """
        scroll_elem = self.driver.run_js("get_scroll_elem(arguments[0])",self.elem,script="scroll.js")

        if method == "pageup":
            scroll_elem.send_keys(Keys.PAGE_UP)
        if method == "pagedown":
            scroll_elem.send_keys(Keys.PAGE_DOWN)



import os
import fire
import time


class UiTester(object):
    def init(self):
        print(123)
    def start(self,filename):
        os.system("pytest " + filename)

    def server(self,order):
        if order == "start":
            os.system("selenium-standalone start")
        if order == "install":
            install_standalone = "sudo npm install selenium-standalone@latest -g --registry=https://registry.npm.taobao.org"
            install_dirver = "sudo selenium-standalone install --drivers.firefox.baseURL=http://npm.taobao.org/mirrors/geckodriver --baseURL=http://npm.taobao.org/mirrors/selenium --drivers.chrome.baseURL=http://npm.taobao.org/mirrors/chromedriver --drivers.ie.baseURL=http://npm.taobao.org/mirrors/selenium"
            os.system(install_standalone + " && " + install_dirver)

def cli():
    fire.Fire(UiTester)


import time
import os
import json
from selenium import webdriver
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
from selenium.webdriver.common.proxy import Proxy
from selenium.webdriver.common.keys import Keys
from .common import Logger,data,Singleton,timethis
from .element import WebElement

class WebDriver(metaclass=Singleton):

    def __init__(self):
        options = webdriver.ChromeOptions()
        options.add_argument("--start-maximized")
        options.add_experimental_option('excludeSwitches', ['enable-automation'])
        self.logger = Logger("pywebdriver")
        self.driver = webdriver.Remote(command_executor='http://127.0.0.1:4444/wd/hub',
            desired_capabilities=DesiredCapabilities.CHROME,
            options=options)
        self.driver.run_js = self.run_js


    def run_js(self,func,*args,script="query.js"):
        """
        调用avascript文件内的函数
        ARGS:
            * func: 需要调用的函数（含参数）
            * args: 要传入js的参数
            * scrip: 脚本文件名称

        USAGE:
            * self.run_js("query_by_locator('text','left')")
            * self.run_js("get_scroll_elem(arguments[0])",elem,script="scroll.js")
        """
        script_path = os.path.join(os.path.dirname(os.path.abspath(__file__)),"javascript",script)

        with open(script_path,'r',encoding='UTF-8') as f:
            js = f.read() + '\n\n\n' + 'return ' + func

        return self.driver.execute_script(js,*args)

    @timethis
    def query(self,innerText=None,locator=None,locator_text=None,css=None,parts=None,timeout=100,index=0,exit=True,**kwargs):
        '''
        返回选中的元素，并保存选中的元素到self.elem
        ARGS:
            * INNERTEXT 要定位的元素的文字
            * EXIT 没有找到元素后是否退出
                * True: 退出并报错
                * False: 不退出并return False
        '''
        def query_elem(innerText,css):
            if locator:
                if parts:
                    elem_list = self.run_js("query_by_part_of_locator('{}','{}')".format(innerText,locator))
                else:
                    if locator_text == None:
                        elem_list = self.run_js("query_by_locator('{}','{}')".format(innerText,locator))
                    else:
                        elem_list = self.run_js("query_by_locator('{}','{}','{}')".format(innerText,locator,locator_text))
            elif parts:
                elem_list = self.run_js("query_by_parts_of_innerText('{0}')".format(innerText))
            elif innerText:
                elem_list = self.run_js("query_by_innerText('{0}')".format(innerText))
            elif css:
                elem_list = self.run_js("query_by_css('{0}')".format(css))
            # 属性选择器
            else:
                key = [*kwargs][0]
                elem_list = self.run_js("query_by_css('[{} = {}]')".format(key,kwargs[key]))
                self.prop_msg = '{} = {}'.format(key,kwargs[key])
            return elem_list

        start_time = time.time()
        end_time = time.time()

        while (end_time - start_time) < timeout:
            elem_list = []
            elem_list += query_elem(innerText,css)
            if elem_list:
                print('FOUND ELEM ' + (innerText or css or self.prop_msg),end = "   ")
                self.logger.debug('FOUND ELEM ' + (innerText or css or self.prop_msg))
                return WebElement(elem_list[index],self.driver)
            end_time = time.time()
            time.sleep(1)
        if exit:
            self.driver.save_screenshot('ERROR.png')
            error_msg = 'can not find Element {} in {} s'.format(innerText or css or parts,timeout)
            self.logger.error('ERROR: ' + error_msg)
            raise Exception(error_msg)
        else:
            return False


    def query_not(self,*args,**kwargs):
        try:
            self.query(*args,timeout = 1,**kwargs)
        except Exception as e:
            self.logger.info("ELEM NOT FOUND : PASS")
            return True
        else:
            raise Exception("ELEM FOUND ERROR")

    @timethis
    def query_part(self,innerText = None,locator = None,timeout = 60,index = 0):
        '''
        select elems by part of innertText
        '''
        return self.query(innerText = innerText,locator = locator,parts = True,timeout = timeout,index = index)


    def diff(self,data = None):
        # 获取页面是否发生了变化，返回 None 或者 一个字典
        if not data:
            result = self.run_js("getElementStyleMD5()")
        else:
            result = self.run_js("getDiffElement('{}')".format(data))

        if not result:
            return None
        else:
            return json.loads(result)

    def get(self,url):
        self.logger.info("GET URL: " + url)
        self.driver.get(url)

    def quit(self):
        self.logger.info("BROWSER CLOSE")
        data.is_quit = True
        self.driver.quit()

    def sleep(self,sleep_time):
        self.logger.info("BROWSER SLEEP: " + str(sleep_time))
        time.sleep(sleep_time)

    def open(self,url):
        self.logger.info("OPEN NEW TAP: " + url)
        script = 'window.open("{}")'.format(url)
        self.driver.execute_script(script)
        #移动句柄，对新打开页面进行操作
        # self.driver.switch_to_window(driver.window_handles[1])
        # #具体操作
        # driver.find_element_by_xpath("")
        # #关闭该新打开的页面
        # driver.close()
        # #不关闭，要移动到上一个页面，我们要移动句柄
        # driver.switch_to_window(driver.window_handles[0])
    def delete_all_cookies(self):
        self.logger.info("CLEAR ALL COOKIES")
        self.driver.delete_all_cookies()



from .data import data

class Singleton(type):
    """
    单例类
    如果 data.is_quit == True 或者 第一次初始化，则返回新的实例
    否则 返回类的拷贝
    """
    def __init__(self, *args, **kwargs):

        self.__instance = None
        super().__init__(*args, **kwargs)

    def __call__(self, *args, **kwargs):
        if self.__instance is None or data.is_quit:
            data.is_quit = False
            self.__instance = super().__call__(*args, **kwargs)
            return self.__instance

        else:
            return self.__instance




import codecs
import os
import sys
try:
  from setuptools import setup,find_packages
except:
  from distutils.core import setup

def read(fname):
  return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
  name = "uitester",
  version = "0.2",
  description = "package description.",
  long_description = read("README.rst"),
  classifiers =  ['License :: OSI Approved :: MIT License','Programming Language :: Python','Intended Audience :: Developers','Operating System :: OS Independent'],
  keywords = "web ui autotest python package",
  author = "gaianote311",
  author_email = "gaianote311@gmail.com",
  url = "http://gaianote.github.io/",
  license = "MIT",
  packages = find_packages(),
  include_package_data=True,
  package_data = {'uitester':['javascript/*']},
  zip_safe=True,
  install_requires = ["selenium","fire","pytest"],
  entry_points={
        'console_scripts':[
        'uitester = uitester:cli'
    ]}
)



from functools import wraps
import time
def timethis(func):
    """
    计时功能，计算一个函数运行所需的适合
    """
    @wraps(func)
    def wrapper(*args, **kwargs):
        start = time.time()
        result = func(*args, **kwargs)
        end = time.time()
        print("{:.2f}s".format(end-start))
        return result
    return wrapper